
groupCounter = 100
unitCounter = 100

function DropoffGroupDirect(typeList, radius, xCenter, yCenter, xDest, yDest)
    local group = {
        ["visible"] = false,
        ["taskSelected"] = true,
        ["groupId"] = groupCounter,
        ["hidden"] = false,
        ["units"] = {},
        ["y"] = yCenter,
        ["x"] = xCenter,
        ["name"] = "GroupName" .. groupCounter,
        ["start_time"] = 0,
        ["task"] = "Ground Nothing",
        ["route"] = {
            ["points"] = 
            {
                [1] = 
                {
                    ["alt"] = 41,
                    ["type"] = "Turning Point",
                    ["ETA"] = 0,
                    ["alt_type"] = "BARO",
                    ["formation_template"] = "",
                    ["y"] = yCenter,
                    ["x"] = xCenter,
                    ["ETA_locked"] = true,
                    ["speed"] = 5.5555555555556,
                    ["action"] = "Diamond",
                    ["task"] = 
                    {
                        ["id"] = "ComboTask",
                        ["params"] = 
                        {
                            ["tasks"] = 
                            {
                            }, -- end of ["tasks"]
                        }, -- end of ["params"]
                    }, -- end of ["task"]
                    ["speed_locked"] = false,
                }, -- end of [1]
                [2] = 
                {
                    ["alt"] = 54,
                    ["type"] = "Turning Point",
                    ["ETA"] = 52.09716824195,
                    ["alt_type"] = "BARO",
                    ["formation_template"] = "",
                    ["y"] = yDest,
                    ["x"] = xDest,
                    ["ETA_locked"] = false,
                    ["speed"] = 5.5555555555556,
                    ["action"] = "Diamond",
                    ["task"] = 
                    {
                        ["id"] = "ComboTask",
                        ["params"] = 
                        {
                            ["tasks"] = 
                            {
                            }, -- end of ["tasks"]
                        }, -- end of ["params"]
                    }, -- end of ["task"]
                    ["speed_locked"] = false,
                }, -- end of [2]
            }, -- end of ["points"]
        }, -- end of ["route"]
    }

    groupCounter = groupCounter + 1
    
    for i = 1,#typeList do  
        local angle = math.pi * 2 * (i-1) / #typeList
        local xofs = math.cos(angle) * radius
        local yofs = math.sin(angle) * radius
        local unitType = typeList[i]
        group.units[i] = NewSoldierUnit(xCenter + xofs, yCenter + yofs, angle, unitType)        
    end
    
    return group
end

function NewSoldierUnit(x, y, heading, unitType)
    local unit = {
        ["y"] = y,
        ["type"] = unitType,
        ["name"] = "Unitname" .. unitCounter,
        ["unitId"] = unitCounter,
        ["heading"] = heading,
        ["playerCanDrive"] = true,
        ["skill"] = "Excellent",
        ["x"] = x,
    }
    
    unitCounter = unitCounter + 1
    
    return unit    
end


PickupZones = {
    [1] = {
        Name = "the FOB",
        ZoneName = "PickupZone",
        SmokeColor = trigger.smokeColor.Blue,
    },
}

DropoffZones = {
    [1] = {
        Name = "Nuclear Plant (Red)",
        ZoneName = "DropoffZone1",
        SmokeColor = trigger.smokeColor.Red,
        DropFunction = DropoffGroupDirect
    },
    [2] = {
        Name = "Rail Station (Orange)",
        ZoneName = "DropoffZone2",
        SmokeColor = trigger.smokeColor.Orange,
        DropFunction = DropoffGroupDirect
    },
    [3] = {
        Name = "City Center (Green)",
        ZoneName = "DropoffZone3",
        SmokeColor = trigger.smokeColor.Green,
        DropFunction = DropoffGroupDirect
    },
    [4] = {
        Name = "Hospitals (Blue)",
        ZoneName = "DropoffZone4",
        SmokeColor = trigger.smokeColor.Blue,
        DropFunction = DropoffGroupDirect
    },
}

function SpawnSmoke(smokeX, smokeY, smokeColor)
    local pos2 = { x = smokeX, y = smokeY }
    local alt = land.getHeight(pos2)
    local pos3 = {x=pos2.x, y=alt, z=pos2.y}
    trigger.action.smoke(pos3, smokeColor)
end

function SmokeTimer(arg, time)    
    for i=1,#PickupZones do
       local zone = trigger.misc.getZone(PickupZones[i].ZoneName)
       SpawnSmoke(zone.point.x, zone.point.z, PickupZones[i].SmokeColor)
    end
    
    for i=1,#DropoffZones do
       local zone = trigger.misc.getZone(DropoffZones[i].ZoneName)
       SpawnSmoke(zone.point.x, zone.point.z, DropoffZones[i].SmokeColor)
    end
    
    return time + 270
end

function FillUnitComposition()
    local retval = {}
    for i=1,3 do
        retval[i] = "Soldier M249"
    end
    for i=4,11 do
        retval[i] = "Soldier M4"
    end
    
    return retval
end

function FindNearestPickupGroup(playerUnit, maxDistance)
    local retval = {
        Group = nil,
        Composition = {}
    }
    
    local playerpos = playerUnit:getPoint()
    
    local minDist = maxDistance
    
    local groups = coalition.getGroups(coalition.side.BLUE, Group.Category.GROUND)
    for i=1,#groups do
        local group = groups[i]
        if group ~= nil then
            local units = group:getUnits()
            local comp = {}
            local groupMin = maxDistance
            
            for j=1,#units do
                local unit = units[j]
                if unit ~= nil then
                    local unitpos = unit:getPoint()
                    local typename = unit:getTypeName()
                    if typename == "Soldier M249" or typename == "Soldier M4" then                        
                        table.insert(comp, typename)
                        
                        local dist = GetDistance(unitpos.x, unitpos.z, playerpos.x, playerpos.z)
                        if dist < groupMin then
                            groupMin = dist
                        end
                        
                    end
                end
                
            end
            
            if groupMin < minDist then
                minDist = groupMin
                retval.Composition = {}
                for i=1,#comp do
                    retval.Composition[i] = comp[i]
                end
                retval.Group = group            
            end
            
        end
    end
    
    return retval
end

UnitStateTable = {}


function UnitRadioCommand(unitName)
    local unit = Unit.getByName(unitName)
    
    if unit == nil then
        UnitStateTable[unitName] = {}
        return
    end
    
    local unitId = unit:getID()
    local group = unit:getGroup()
    local groupName = group:getName()
    local playerName = unit:getPlayerName()
    
    if UnitStateTable[unitName] == nil then
        UnitStateTable[unitName] = {}
        
        if unitName == "HueyPilot9" then
            UnitStateTable[unitName] = FillUnitComposition()
        end
    end
    
    local pickupZone = UnitInAnyPickupZone(unit)
    local dropoffZone = UnitInAnyDropoffZone(unit)
    
    if #UnitStateTable[unitName] > 0 then
        
        -- if in pickupZone (fob) then drop troops and credit score
        -- if in dropoff zone, then drop the troops in place
        -- if neither
        
        if dropoffZone ~= nil then
            local unitpos = unit:getPoint()
            local triggerZone = trigger.misc.getZone(dropoffZone.ZoneName)
            local newGroup = dropoffZone.DropFunction(UnitStateTable[unitName], 15, unitpos.x, unitpos.z, triggerZone.point.x, triggerZone.point.z)
            coalition.addGroup(country.id.USA, Group.Category.GROUND, newGroup)
            
            trigger.action.outText(playerName .. " (" .. groupName .. ") dropped " .. #UnitStateTable[unitName] .. " soldiers at " .. dropoffZone.Name, 10)
            
            UnitStateTable[unitName] = {}
        else
            if pickupZone ~= nil then
                trigger.action.outText(playerName .. " (" .. groupName .. ") returned " .. #UnitStateTable[unitName] .. " soldiers to " .. pickupZone.Name, 10)
                UnitStateTable[unitName] = {}
            else
                trigger.action.outText(playerName .. " (" .. groupName .. ") isn't in a pickup or dropoff zone. (" .. #UnitStateTable[unitName] .. " soldiers aboard)", 10)
            end
        end
    else
        -- we're empty
        
        -- if we're in pickup zone, load troops
        if pickupZone ~= nil then
            UnitStateTable[unitName] = FillUnitComposition()
            trigger.action.outText(playerName .. " (" .. groupName .. ") loaded a squad at " .. pickupZone.Name .. ".", 10)
        else
            if dropoffZone ~= nil then
                -- check for troops to load
                local pickupResult = FindNearestPickupGroup(unit, 30)
                if pickupResult.Group ~= nil then
                    pickupResult.Group:destroy()
                    UnitStateTable[unitName] = pickupResult.Composition
                    trigger.action.outText(playerName .. " (" .. groupName .. ")  evacuated a squad of " .. #pickupResult.Composition .. " from " .. dropoffZone.Name , 10)
                else
                    trigger.action.outText(playerName .. " (" .. groupName .. ") must land within 30m of a squad for evac pickup.", 10)
                end
            else
                trigger.action.outText(playerName .. " (" .. groupName .. ") isn't in a pickup or dropoff zone. (" .. #UnitStateTable[unitName] .. " soldiers aboard)", 10)
            end
        end
        


    end
           
end

RadioCommandTable = {}

function AddRadioCommand(unitName)
    if RadioCommandTable[unitName] == nil then
        local unit = Unit.getByName(unitName)
        if unit == nil then
            return
        end
        
        local group = unit:getGroup()
        if group == nil then
            return
        end
        
        local gid = group:getID()
        
        missionCommands.addCommandForGroup(gid, "Load/unload Troops", nil, UnitRadioCommand, unitName)
        RadioCommandTable[unitName] = true
    end
end


function AddRadioCommands(arg, time)
    AddRadioCommand("HueyPilot1")
    AddRadioCommand("HueyPilot2")
    AddRadioCommand("HueyPilot3")
    AddRadioCommand("HueyPilot4")
    AddRadioCommand("HueyPilot5")
    AddRadioCommand("HueyPilot6")
    AddRadioCommand("HueyPilot7")
    AddRadioCommand("HueyPilot8") 
    AddRadioCommand("HueyPilot9")
    return time + 5
end

function GetDistance(xUnit, yUnit, xZone, yZone)
    local xDiff = xUnit - xZone
    local yDiff = yUnit - yZone
    return math.sqrt(xDiff * xDiff + yDiff * yDiff)    
end


function FindNearestDropoffZone(unit, maxDistance)
    local minDist = maxDistance
    local minZone = nil
    local unitpos = unit:getPoint()
    
    for i=1,#DropoffZones do
        local zone = DropoffZones[i]
        local triggerZone = trigger.misc.getZone(zone.ZoneName)
        local dist = GetDistance(unitpos.x, unitpos.z, triggerZone.point.x, triggerZone.point.z)
        if dist < minDist then
            minDist = dist
            minZone = zone
        end
    end
    
    return minZone    
end


function StatusReport(arg, time)
    -- array of Group function coalition.getGroups(enum coalition.side coalition, enum Group.Category groupCategory or nil)
    -- array of Unit function Group.getUnits(Group self)
    -- enum country.id CoalitionObject.getCountry(CoalitionObject self)

    local zoneTable = {}
    local zoneFriendly = {}

    for i=1,#DropoffZones do
        zoneTable[DropoffZones[i].Name] = 0
        zoneFriendly[DropoffZones[i].Name] = 0
    end

    local groups = coalition.getGroups(coalition.side.RED, Group.Category.GROUND)
    for i=1,#groups do
        local group = groups[i]
        if group ~= nil then
            local units = group:getUnits()
            for j=1,#units do
                local unit = units[j]
                if unit ~= nil then
                    local country = unit:getCountry()
                    if country == 17 then       -- if INSURGENT
                        local zone = FindNearestDropoffZone(unit, 5000)
                        if zone ~= nil then
                            zoneTable[zone.Name] = zoneTable[zone.Name] + 1
                        end
                    end
                end
            end
        end
    end
    
    local groups = coalition.getGroups(coalition.side.BLUE, Group.Category.GROUND)
    for i=1,#groups do
        local group = groups[i]
        if group ~= nil then
            local units = group:getUnits()
            for j=1,#units do
                local unit = units[j]
                if unit ~= nil then
                    local typename = unit:getTypeName()
                    if typename == "Soldier M249" or typename == "Soldier M4" then
                        local zone = FindNearestDropoffZone(unit, 100)
                        if zone ~= nil then
                            zoneFriendly[zone.Name] = zoneFriendly[zone.Name] + 1
                        end
                    end
                end
            end
        end
    end

    local text = "MISSION STATUS  -  See mission briefing for details\n\n"

    for k,v in pairs(zoneTable) do
        if v > 0 then
            text = text .. tostring(k) .. ": " .. tostring(v) .. " insurgent units remain.\n"
        else
            if zoneFriendly[k] > 0 then
                text = text .. tostring(k) .. ": Friendly units waiting for evac: " .. zoneFriendly[k] .. ".\n"
            else
                text = text .. tostring(k) .. ": Area cleared!\n"
            end
        end
    end
    
    
    trigger.action.outText(text, 20)
    
    return time + 45
end
   

do
    timer.scheduleFunction(SmokeTimer, nil, timer.getTime() + 5)
    timer.scheduleFunction(AddRadioCommands, nil, timer.getTime() + 5)
    timer.scheduleFunction(StatusReport, nil, timer.getTime() + 45)
end


function UnitInAnyPickupZone(unit)
    for i=1,#PickupZones do
        if UnitInZone(unit, PickupZones[i]) then
            return PickupZones[i]
        end
    end
    
    return nil
end

function UnitInAnyDropoffZone(unit)
    for i=1,#DropoffZones do
        if UnitInZone(unit, DropoffZones[i]) then
            return DropoffZones[i]
        end
    end
    
    return nil
end


function UnitInZone(unit, zone)
    if unit:inAir() then
        return false
    end
    
    local triggerZone = trigger.misc.getZone(zone.ZoneName)
    local group = unit:getGroup()
    local groupid = group:getID()
    local unitpos = unit:getPoint()
    local xDiff = unitpos.x - triggerZone.point.x
    local yDiff = unitpos.z - triggerZone.point.z
    local dist = math.sqrt(xDiff * xDiff + yDiff * yDiff)
    
    if dist > triggerZone.radius then
        return false
    end
    
    return true
end


